//
// $Id:$
// Copyright 2007, Ralph Ronnquist
//
// This is a trend indicator, with three lines

#property copyright "Copyright 2007, Ralph Ronnquist"
#property link      "www.richplank.com"

#property indicator_chart_window
#property indicator_buffers 7
#property indicator_color1 Blue
#property indicator_color2 Red
#property indicator_color3 Silver
#property indicator_color4 Red
#property indicator_color5 Blue

#property indicator_width4 2
#property indicator_width5 2

extern int base = 1;        // Fastest MA
extern int slowest = 200;   // Slowest MA
extern int pips = 0;        // Minimal difference
extern int bend = 40;
extern bool exact = true;

// Display buffers
double above[];
double below[];
double half[];
double dnmark[];
double upmark[];

// Auxillary buffers
double upper[];
double lower[];

int init()
{
    int index = 0;
    SetIndexStyle( index, DRAW_LINE );
    SetIndexBuffer( index, above );
    SetIndexLabel( index, "above" );
    
    index = 1;
    SetIndexStyle( index, DRAW_LINE );
    SetIndexBuffer( index, below );
    SetIndexLabel( index, "below" );
    
    index = 2;
    SetIndexStyle( index, DRAW_LINE );
    SetIndexBuffer( index, half );
    SetIndexLabel( index, "half" );
    
    index = 3;
    SetIndexStyle( index, DRAW_ARROW );
    SetIndexBuffer( index, dnmark );
    SetIndexLabel( index, "down mark" );
    SetIndexArrow( index, 238 );
    
    index = 4;
    SetIndexStyle( index, DRAW_ARROW );
    SetIndexBuffer( index, upmark );
    SetIndexLabel( index, "up mark" );
    SetIndexArrow( index, 236 );
    
    index = 5;
    SetIndexStyle( index, DRAW_NONE );
    SetIndexBuffer( index, upper );
    SetIndexLabel( index, "upper" );

    index = 6;
    SetIndexStyle( index, DRAW_NONE );
    SetIndexBuffer( index, lower );
    SetIndexLabel( index, "lower" );

    return(0);
}

int deinit()
{
    return(0);
}

int start()
{
    for ( int bar = Bars - IndicatorCounted(); bar >= 0; bar-- ) {
        indicate( bar );
    }
    return(0);
}

void indicate(int bar)
{
    above[ bar ] = EMPTY_VALUE;
    below[ bar ] = EMPTY_VALUE;
    upmark[ bar ] = EMPTY_VALUE;
    dnmark[ bar ] = EMPTY_VALUE;

    // Actual computation requires at least 'slowest' number of bars.
    // Before then, the marking is for 'ambiguous', and display
    // buffers are cleared
    if ( Bars - bar < slowest ) {
        upper[ bar ] = slowest;
        lower[ bar ] = slowest;
        return( 0 );
    }

    // Compute the periods for the first slow MAs that differ from the
    // base MA by +/- edge. That is, the fastest MA period where the
    // base MA is above or below that MA by at least the edge amount
    // (given in pips).

    int a = slowest;    // Period for slowest up turn MA
    int b = slowest;    // Period for slowest down turn MA
    double edge = pips * Point;

    double s = sum( base, bar );
    double f = s / base;

    for ( int p = base + 1; p < slowest; p++ ) {
        s += Open[ p + bar ];
        double d = NormalizeDouble( f - s / p, Digits );
        if ( d > edge ) {
            if ( b == slowest )
                b = p;
            if ( a != slowest )
                break;
        } else if ( d < -edge ) {
            if ( a == slowest )
                a = p;
            if ( b != slowest )
                break;
        }
    }

    half[ bar ] = NormalizeDouble( 2 * sum( slowest / 2, bar ) / slowest, Digits );
    
    // MA(base) is above MA(a) by edge amount, or 'a' is the slowest MA, and
    // MA(base) is below MA(b) by edge amount, or 'b' is the slowest MA.

    upper[ bar ] = a; // Defining the Up Marker
    lower[ bar ] = b; // Defining the Down Marker

    above[ bar ] = avg( a, bar ) + edge;
    below[ bar ] = avg( b, bar ) - edge;

    if ( turning( lower, bar, bend, slowest ) ) {
        if ( below[ bar ] < below[ bar+1 ] || upmark[ bar+1 ] != EMPTY_VALUE )
            upmark[ bar ] = Low[ bar+1 ];
        else if ( Open[ bar ] > above[ bar ] && upper[ bar ] < slowest )
            dnmark[ bar ] = Low[ bar+1 ]; // Reverse marker
    }

    if ( turning( upper, bar, bend, slowest ) ) {
        if ( above[ bar ] > above[ bar+1 ] || dnmark[ bar+1 ] != EMPTY_VALUE )
            dnmark[ bar ] = High[ bar+1 ];
        else if ( Open[ bar ] < below[ bar ] && lower[ bar ] < slowest )
            upmark[ bar ] = High[ bar+1 ]; // Reverse marker
    }
    
    if ( exact ) {
        return( 0 );
    }

    above[ bar ] = EMPTY_VALUE;
    below[ bar ] = EMPTY_VALUE;

    // The following is a bit of decision logic to determine trend.
    // If neither marker is 'slowest', then promote the marker that
    // was 'slowest' at the previous bar, provided that the other
    // marker wasn't also 'slowest'.

    if ( a < slowest && b < slowest ) {
        if ( upper[ bar+1 ] == slowest && lower[ bar+1 ] < slowest ) {
            a = slowest;
        } else if ( upper[ bar+1 ] < slowest && lower[ bar+1 ] == slowest ) {
            b = slowest;
        }
    }

    // Display logic: show marked MA's with as edge offsets. The blue
    // point tells where the base MA is relative to the Up Marker MA,
    // and the red point does the same for the Down Marker MA. 
    double va = avg( a, bar ) + edge;
    double vb = avg( b, bar ) - edge;

    if ( a == slowest ) {
        above[ bar ] = va;
    }
    if ( b == slowest ) {
        below[ bar ] = vb;
    }
}

bool turning(double &line[],int bar,int bend,int slowest)
{
    int x = line[ bar ];
    if ( x > bend )
        return( false );
    for (int i = bar + 1; i < Bars; i++ ) {
        if ( line[ i ] > bend )
            return( line[ i ] == slowest && ( i - bar > 1 ) );
        if ( line[ i ] <= x )
            break;
        x = line[ i ];
    }
    return( false );
}

/**
 * Compute the sum of Open prices of period bars from given bar
 */
double sum(int period,int bar)
{
    double v = 0;
    for ( int i = bar + period - 1; i >= bar; i-- ) {
        v += Open[ i ];
    }
    return( v );
}

/**
 * Compute the average of Open prices of period bars from given bar
 */
double avg(int period,int bar)
{
    return( sum( period, bar ) / period );
}

//+------------------------------------------------------------------+